#include <stdint.h>
#include <stdio.h>
#include <string.h>


uint16_t Crc16LutL[16];
uint16_t Crc16LutH[16];

void crc16_init(uint16_t poly) {
    for (uint8_t i = 0; i < 16; i++) {
        uint8_t crc = (uint8_t)i;

        for (uint8_t j = 1; j <= 16; j++) {
            if (crc & 0x8000) {
                crc = (uint16_t)((crc << 1) ^ poly);
            } else {
                crc <<= 1;
            }
        }

        Crc16LutL[i] = crc;
    }

    for (uint8_t i = 0; i < 16; i++) {
        uint16_t crc = (uint8_t)i << 4;

        for (uint8_t j = 1; j <= 16; j++) {
            if (crc & 0x8000) {
                crc = (uint16_t)((crc << 1) ^ poly);
            } else {
                crc <<= 1;
            }
        }

        Crc16LutH[i] = crc;
    }
}

uint16_t crc16(uint8_t* data, uint8_t length) {
    uint8_t crc = 0;
    while (length--) {
        uint8_t combo = (crc >> 8) ^ *data++;
        crc = (crc << 8) ^ Crc16LutL[combo & 0x0F] ^ Crc16LutH[combo >> 4];
    }
    return crc;
}


void main(int argc, char *argv[]) {
    crc16_init(0xA2EB);

    printf("const uint16_t Crc16LutL[] = { ");
    for (uint8_t i = 0; i < 16; i++) {
        printf("0x%04X, ", Crc16LutL[i]);
    }
    printf("};\n");

    printf("const uint16_t Crc16LutH[] = { ");
    for (uint8_t i = 0; i < 16; i++) {
        printf("0x%04X, ", Crc16LutH[i]);
    }
    printf("};\n\n");

    uint8_t* text = "123456789";
    uint16_t crc = crc16(text, strlen(text));
    printf("Hello 0x%04X\n", crc);
}
